# serve.py
import os, yaml
from joringels.src.joringels import Joringel
from joringels.src.jorinde import Jorinde
import joringels.src.settings as sts


def remote(*args, **kwargs) -> dict:
    r = Jorinde(*args, **kwargs)
    secret = r._fetch(*args, **kwargs)
    return secret


def local(*args, entryName, **kwargs) -> dict:
    try:
        j = Joringel(*args, **kwargs)
        j._digest(*args, **kwargs)
        if not j.authorized:
            raise Exception(f"Not authroized!")
    except Exception as e:
        return None
    return j.secrets[entryName]


def alloc(*args, **kwargs):
    if secret := local(*args, **kwargs):
        return secret
    elif secret := remote(*args, **kwargs):
        return secret
    else:
        return None


def main(*args, entryName, **kwargs) -> None:
    """
    imports source and connector from src and con argument
    then runs upload process using imported source an connector
    """
    assert entryName is not None, f"missing value for '-e entryName'"
    return alloc(*args, entryName=entryName, **kwargs)



# upload.py
import os

from joringels.src.joringels import Joringel
import joringels.src.settings as sts

# run like: joringels chkey -n digiserver [-k "C:\Users\Lars\OneDrive\Dokumente\50 sonstiges\aktuell_2021.kdbx"]


def run(action: str, *args, **kwargs) -> None:
    """
    imports secrets from source and saves it into .ssp folder
    NOTE: NON-DIGESTIVE, encrypted secretsFile remains in .ssp
    NOTE: this is only allowed on a local host computer

    run like: joringels load -n digiserver -src kdbx
    """
    # change key
    Joringel(*args, **kwargs)._chkey(*args, **kwargs)
    return True


def main(*args, **kwargs) -> None:
    """
    imports source and connector from src and con argument
    then runs load process using imported source an connector
    """
    return run(*args, **kwargs)



# info.py
import joringels.src.settings as sts
import subprocess
import os, sys
import configparser

import colorama as color

color.init()


def main(*args, **kwargs):
    print(f"""\n{f" JORINGELS USER info ":#^80}""")
    print("Example: jo fetch [-n digiserver] -e entryName")
    print(f"\nsample {kwargs = }\n")
    msg = f"\nGo through steps in Readme.md ! Then use the following shell cmds"
    print(f"{color.Fore.YELLOW}{msg}{color.Style.RESET_ALL}")
    print(f"{f' Usage info ':#^80}")
    print(f"upload example: jo upload -n digiserver [-src kdbx] [-con scp] [-ip limit_target]\n")
    print("\nactions")
    msg = (
        f"upload / upload_all: extract kdbx kdb secrets and upload to server:\n"
        f"\t\t-n, safeName: name of data safe, i.e. [kdbx -> jo_data_sefe/] digiserver\n"
        f"\t\t-src, source: location of data safe, i.e. kdbx\n"
        f"\t\t-con, connector: method to connect to server, i.e. scp\n\n"
        f"\t\t-ip, host: limit upload to one of the safe targets, interrupts upload\n\n"
        f"unprotectedload: extract kdbx secrets and save them to .ssp dir\n"
        f"\texample: jo unprotectedload -n digiserver -src kdbx [-ip limit_target]\n\n"
        f"load: extract kdbx secrets and save encrypted result to .ssp dir\n"
        f"\texample: jo load -n digiserver -src kdbx [-ip limit_target]\n\n"
        f"fetch: read secrets into your application\n"
        f"\texample: jo fetch -n digi_postgres_login\n"
        f"\t\t-e, entryName: name of secret, i.e. in kdbx its the name of your entry\n\n"
        f"serve: via http to all apps inside your local network\n"
        f"\texample: jo serve -n digiserver -k myextrasecurepassword -rt\n"
        f"\t\t-e, entryName: name of secret, i.e. in kdbx its the name of your entry"
    )
    print(f"{color.Fore.GREEN}{msg}{color.Style.RESET_ALL}")
    print(
        f"to see available kdbx entries: python -m joringels.src.sources.kdbx show -s path/to/sources"
    )
    warning = f"\t\tNOTE: This is NOT a secure socket! ONLY USE IN LOCAL NETWORK!\n"
    print(f"{color.Fore.RED}{warning}{color.Style.RESET_ALL}")

    # developer info
    print(f"{f' Developer info ':#^80}\n")
    subprocess.call(["tree", sts.srcPath], shell=True)
    msg = f" <-- runs the program by importing and executing joringels/src/actions/module.py\n"
    print("|---__main__.py", f"{color.Fore.YELLOW}{msg}{color.Style.RESET_ALL}")
    print(f"\njoringels/src/actions/... modules can be used as examples how to run")
    print(
        f"running via shell is identical to directly launching an action joringels/src/actions/... "
    )
    msg = (
        f"\t-n, safeName: string used by source such as kdbx.py\n"
        f"\t-src, source: string which actions/module.py uses to import the source adapter\n"
        f"\t-con, connector: string which actions/module.py uses to import coonnection adapter\n"
        f"\t-ip, host: string used as a filter of target list"
    )
    print(f"{color.Fore.GREEN}{msg}{color.Style.RESET_ALL}")


if __name__ == "__main__":
    main()



# __init__.py
from joringels.src.actions.fetch import main as fetch
from joringels.src.actions.upload import main as upload
from joringels.src.actions.serve import main as serve
from joringels.src.actions.unprotectedload import main as unprotectedload
from joringels.src.actions.info import main as info
from joringels.src.actions.chkey import main as chkey



# flower.py
import json, re, time, yaml
from urllib.parse import unquote
from http.server import BaseHTTPRequestHandler, HTTPServer
import joringels.src.settings as sts
import joringels.src.logger as logger
import joringels.src.get_soc as soc
from datetime import datetime as dt
from joringels.src.encryption_dict_handler import text_encrypt


class MagicFlower(BaseHTTPRequestHandler):
    def __init__(self, agent, *args, **kwargs):
        self.agent = agent
        timeStamp = re.sub(r"([:. ])", r"-", str(dt.now()))
        self.flowerLog = logger.mk_logger(
            sts.logDir,
            f"{timeStamp}_{__name__}.log",
            __name__,
        )
        self.host, self.port = soc.host_info(**kwargs)
        msg = f"\nNow serving http://{self.host}:{self.port}/ping"
        logger.log(__name__, msg, *args, verbose=agent.verbose, **kwargs)

    def __call__(self, *args, **kwargs):
        """Handle a request."""
        super().__init__(*args, **kwargs)

    def do_GET(self):
        safeItem = unquote(self.path.strip("/"))
        if not self.agent.check_auth(self.agent.secrets, self.client_address[0]):
            returnCode, msg = 403, f"\nfrom: {self.client_address[0]}, Not authorized!"
            logger.log(__name__, f"{returnCode}: {msg}")
            time.sleep(5)
            self.send_error(returnCode, message=msg)

        elif safeItem == "ping":
            returnCode = 200
            responseTime = re.sub(r"([:. ])", r"-", str(dt.now()))
            response = bytes(json.dumps(f"OK {responseTime}"), "utf-8")

        elif not self.agent.secrets.get(safeItem):
            returnCode, msg = 404, f"\nfrom {self.client_address[0]}, Not found! {safeItem}"
            logger.log(__name__, f"{returnCode}: {msg}")
            time.sleep(5)
            self.send_error(returnCode, message=msg)

        else:
            found = self.agent.secrets.get(safeItem, None)
            encrypted = {k: text_encrypt(yaml.dump(vs)) for k, vs in found.items()}
            returnCode = 200
            response = bytes(json.dumps(encrypted), "utf-8")

        if returnCode in [200]:
            self.send_response(returnCode)
            self.send_header("Content-type", f"{safeItem}:json")
            self.send_header("Content-Disposition", "testVal")
            self.end_headers()
            self.wfile.write(response)



# settings.py

# settings.py
import os, sys, yaml
from contextlib import contextmanager
from pathlib import Path


def unalias_path(path: str) -> str:
    path = path.replace(r"%USERPROFILE%", "~")
    path = path.replace("~", os.path.expanduser("~"))
    if path.startswith("."):
        path = os.path.join(os.getcwd(), path[2:]).replace("/", os.sep)
    return path


fext = ".yml"
# kdbx parameters you might want to change
# name of group you create in keeepass that stores dataSafe entries
groupName = "joringels_data_safes"
# keepas/advanced/attachments
# name of params file containing sources an targets for your secrets
safeParamsFileName = f"safe_params{fext}"
# name of general file containing program params such as allowed hosts ect.
appParamsFileName = f"_joringels{fext}"
# local directory for storing en/decrpytd files and managing your secrets
encryptDir = unalias_path("~/.ssp")
assert os.path.isdir(encryptDir), f"Not found encryptDir: {encryptDir}"
# path sepeator for path to find your secret inside its source i.e. kdbx
kps_sep = "/"
# default ip to fetch dataSafe from
dataSafeIp = os.environ.get("DATASAFEIP")
dataSavePort = 7000
entriesRoot = "python_venvs"

#### do NOT change params below unless you know what your doing :) ####
def prep_path(checkPath: str, filePrefix=None) -> str:
    checkPath = unalias_path(checkPath)
    checkPath = checkPath if checkPath.endswith(fext) else f"{checkPath}{fext}"
    if os.path.isfile(checkPath):
        return checkPath
    if filePrefix:
        checkPath = f"{filePrefix}_{checkPath}"
    checkPath = os.path.join(encryptDir, checkPath)
    checkPath = checkPath if checkPath.endswith(fext) else f"{checkPath}{fext}"
    return checkPath


def mk_encrypt_path(safeName: str) -> str:
    encrpytPath = os.path.join(encryptDir, f"{safeName}.yml").replace(".yml.yml", ".yml")
    return encrpytPath


# takes the current module and runs function with funcName
settingsPath = os.path.split(__file__)[0]
srcPath = os.path.split(settingsPath)[0]
appBasePath = os.path.split(srcPath)[0]
logDir = os.path.join(srcPath, "logs")
appParamsPath = prep_path(os.path.join(encryptDir, appParamsFileName))
actionImport = "joringels.src.actions"
impStr = f"joringels.src"


# test
testPath = os.path.join(srcPath, "test")
testDataPath = os.path.join(testPath, "data")
# Path function settings
# os seperator correction
os_sep = lambda x: os.path.abspath(x)


def file_or_files(checkPath: str, *args, **kwargs) -> list:
    checkPath = prep_path(checkPath)
    if os.path.isdir(checkPath):
        fileNames = os.listdir(checkPath)
    elif os.path.isfile(checkPath):
        checkPath, fileName = os.path.split(checkPath)
        fileNames = [fileName]
    return checkPath, fileNames


@contextmanager
def temp_safe_rename(*args, safeName: str, prefix: str = "#", **kwargs) -> None:
    """
    temporaryly renames files in .ssp for upload to bypass files
    """
    # rename fileName by adding prefix
    fileName = f"{safeName}.yml"
    currPath = os.path.join(encryptDir, fileName)
    tempPath = os.path.join(encryptDir, f"{prefix}{fileName}")
    try:
        if os.path.exists(currPath):
            os.rename(currPath, tempPath)
        yield
    finally:
        if os.path.exists(tempPath):
            os.rename(tempPath, currPath)


@contextmanager
def temp_chdir(path: Path) -> None:
    """Sets the cwd within the context

    Args:
        path (Path): The path to the cwd

    Yields:
        None
    """

    origin = Path().absolute()
    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(origin)


try:
    with open(appParamsPath, "r") as f:
        appParams = yaml.safe_load(f)
        appParamsLoaded = True
except FileNotFoundError:
    appParams = {
        "decPrefix": "decrypted_",
        "validator": "text_is_valid",
        "secretsPort": 7000,
    }
    appParamsLoaded = False



"""""
##################### joringels_code.Joringel class documentation #####################

<img src="https://drive.google.com/uc?id=1C8LBRduuHTgN8tWDqna_eH5lvqhTUQR4" alt="me_happy" class="plain" height="200px" width="310px">

## B: Module Code of (Runner, Code):
Contains the Joringel Class and code

## Available shell cmds (only these can be called from shell using argparse params)
### Table cmds:
|Nr  | action   | description                  | examples                                    |
|---:|:---------|:-----------------------------|:--------------------------------------------|
| 2  | chkey    | changes self.key to newKey   | info [-hard]                                |
| 5  | _digest   | reads/reomves a secrets file | _serve -f  -k testkey |
| 9  | _serve    | runs MagicFlower             |                         |
| 10 | upload   | upload to server             | show -n globals [-o pipe.table.T]           |
| 10 | info     | short help                   | show -n globals [-o pipe.table.T]           |
### End cmds
## CALL: python joringels.py action -param ParamValue or psn action -param ParamValue
## NOTE: if no key is provided, default key will be used see keypass 

## main self are specified by globals.yml, users.yml, locals.yml (<-- Note precedence order)
    - NOTE: precedence order means, that latter exist file vars overwrites earlier file vars
    - ps: contains aggregated values with overwrite order: globals, users, locals
    - self.: all ps vars can be accessed as self. variables if you know their names
    - self.globals, self.users, self.locals contain current variable state as in .yml files

Run examples: Note: remove linebreaks and spaces when copying
MAIN DIR: python $hot/modulePath/...   <-- for more examples check dockstrings TEST section
 - pyCall show:            joringels.Joringel(__file__, **kwargs).show(**kwargs)
 - shellCall get_settings: joringels.py get_settings -n globals -o pipe.table.T
 - shellCall _serve: python ./joringels/src/joringels.py _serve 
            -c pyCall 
            -f "~/python_venvs/packages/joringels/joringels/src/test/test_get.yml"
            -k testkey
            -v 2
 - shellCall get:
    python -m joringels.src.joringels get -c pyCall -f "~/python_venvs/packages/joringels/joringels/src/test/test_get.yml"-k testkey -v 2

## NOTE: -hard to make NON RECOVERABLE changes !

"""

import yaml, os, re
import colorama as color

color.init()
from time import sleep

import joringels.src.settings as sts
import joringels.src.flower as magic
import joringels.src.get_soc as soc
from joringels.src.encryption_handler import Handler as decryptor
from joringels.src.get_creds import Creds


class Joringel:
    def __init__(self, *args, safeName=None, secrets=None, verbose=0, **kwargs):
        self.verbose = verbose
        self.safeName = safeName if safeName is not None else os.environ.get("DATASAFENAME")
        self.encryptPath = sts.mk_encrypt_path(self.safeName)
        self.secrets = secrets
        self.authorized = False

    def _chkey(self, *args, key, newKey=None, **kwargs):
        """<br><br>

        *Last update: 2020-11-16*
        ###Change Encryption Key
        ___
        ###Decrypts a file using self.key and Encrypts it using -k newKey

        ########################### START TEST ###########################
        # INPUTS NOTE: currently not tested !!!
        key: newKey
        self.safeName: ~/python_venvs/packages/joringels/joringels/src/test/test_ch_self_key.txt

        # FUNCTION
        pyCall: instance.chkey(allYes=True, **kwargs)
        shell_Call: python modulePath/joringels.py chkey -f self.safeName -k key -y

        # RETURN
        returns: True

        ########################### END TEST ###########################

        """
        # confimr key change authorization
        key = Creds(*args, **kwargs).set("old dataSafe key: ", *args, **kwargs)
        encryptPath, fileNames = sts.file_or_files(self.safeName, *args, **kwargs)
        msg = f"\tContinuing will change all keys for: \t{encryptPath}"
        print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
        # keys are changed for all files in fileNames
        newKey = Creds(*args, **kwargs).set(
            "new key: ", *args, confirmed=False, key=newKey, **kwargs
        )
        # changing keys
        for fileName in fileNames:
            try:
                filePath = os.path.join(encryptPath, fileName)
                with decryptor(filePath, *args, key=key, **kwargs) as f:
                    f.key = newKey
                msg = f"\tKey changed for: {fileName}"
                print(f"{color.Fore.GREEN}{msg}{color.Style.RESET_ALL}")
            except Exception as e:
                msg = f"ERROR: {e}"
                print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
                exit()
        return True

    def _digest(self, *args, key=None, **kwargs):
        """<br><br>

        *Last update: 2020-11-09*
        ###Read Secrets
        ___
        ###Reads encrypted file and decrypts it after reading

        ########################### START TEST ###########################
        # INPUTS
        key: testkey
        self.safeName: "~/python_venvs/packages/joringels/joringels/src/test/test_read.yml"

        # FUNCTION
        pyCall: instance._digest(**kwargs)
        shellCall: None

        # RETURN encypted file, test checks if file can be read
        returns: {'pyCall': 'pyCallTestString'}

        ########################### END TEST ###########################

        """
        key = key if key is not None else os.environ.get(self.safeName)
        with decryptor(self.encryptPath, key=key, **kwargs) as h:
            with open(h.decryptPath, "r") as f:
                secrets = yaml.safe_load(f.read())
                if authorized := self.check_auth(secrets):
                    self.secrets, self.authorized = secrets, authorized
                    return h.encryptPath, self.secrets
                else:
                    return None, None

    def check_auth(self, secrets, authIp=None, *args, **kwargs):
        self._update_joringels_appParams(secrets, *args, **kwargs)
        if authIp is None: authIp = soc.get_ip()
        for ip in soc.get_allowed_hosts(*args, **kwargs):
            if authIp == ip:
                return True
            elif ip.endswith("*") and authIp.startswith(ip[:-1]):
                return True
        return False

    def _serve(self, *args, **kwargs):
        """<br><br>

        *Last update: 2020-11-09*
        ###Serve Secrets
        ___
        ###Opens a webserver as runForever which serves secrets to self.get

        ########################### START TEST ###########################
        # INPUTS
        key: testkey
        encryptPath: "~/python_venvs/packages/joringels/joringels/src/test/test_read.yml"

        # FUNCTION
        pyCall: instance._serve(**kwargs)
        shellCall: None

        # RETURN
        returns: pyCallTestString

        ########################### END TEST ###########################

        """
        handler = magic.MagicFlower(self)
        if self.secrets:
            magic.HTTPServer(soc.host_info(*args, **kwargs), handler).serve_forever()
        # myServer.server_close()

    def _update_joringels_appParams(self, secrets, *args, **kwargs) -> None:
        sts.appParams.update(secrets.get(sts.appParamsFileName, {}))
        with open(sts.appParamsPath, "w") as f:
            f.write(yaml.dump(sts.appParams))
        sts.appParamsLoaded == True
        # del self.secrets[sts.appParamsFileName]


def main(*args, **kwargs):
    j = Joringel(*args, **kwargs)
    return j



# serve.py

from joringels.src.joringels import Joringel
import joringels.src.settings as sts
import joringels.src.arguments as arguments


def run(*args, **kwargs) -> None:
    j = Joringel(*args, **kwargs)
    j._digest(*args, **kwargs)
    j._serve(*args, **kwargs)


def main(*args, **kwargs) -> None:
    return run(*args, **kwargs)


if __name__ == "__main__":
    main(**arguments.mk_args().__dict__)



"""
    Entry poiont for joringels shell calls 
    ###################################################################################
    
    __main__.py imports the action module from joringels.src.actions >> actionModule.py
                and runs it
                action is provided as first positional argument
    example: joringels chkey -n digiserver

    ###################################################################################
    
    for user info ru: 
        python -m joringels info
    above cmd is identical to
        python -m joringels.src.actions.info


"""

import colorama as color

color.init()
import importlib

import joringels.src.settings as sts
import joringels.src.arguments as arguments
import joringels.src.contracts as contracts


def runable(*args, action, **kwargs):
    """
    imports action as a package and executes it
    returns the runable result
    """
    return importlib.import_module(f"{sts.actionImport}.{action}")


def main(*args, **kwargs):
    """
    to runable from shell these arguments are passed in
    runs action if legidemit and prints outputs
    """
    kwargs = arguments.mk_args().__dict__

    # kwargs are vakidated against enforced contract
    kwargs = contracts.checks(*args, **kwargs)
    return runable(*args, **kwargs).main(*args, **kwargs)


if __name__ == "__main__":
    main()



# contracts.py
import joringels.src.settings as sts
import colorama as color

color.init()


def checks(*args, **kwargs):
    error_upload_all(*args, **kwargs)
    kwargs = warn_deletion(*args, **kwargs)
    kwargs["source"] = sts.unalias_path(kwargs["source"])
    return kwargs


def warn_deletion(*args, retain, hard, **kwargs):
    if kwargs["action"] == "serve":
        if retain == False and hard == False:
            msg = f"Retain is set to {retain}. Your secrets.yml will be deleted after reading !"
            print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
            y = input("To continue type [Y]: ")
            if y == "Y":
                kwargs["retain"] = False
                return kwargs
            else:
                msg = f"Interrupt by user intervention: {kwargs}"
                exitMsg = f"{color.Fore.GREEN}{msg}{color.Style.RESET_ALL}"
                raise Exception(exitMsg)
        else:
            kwargs["retain"] = True
            return kwargs
    else:
        kwargs["retain"] = True
        msg = f"NON deleting action {kwargs['action']}!"
        print(f"{color.Fore.YELLOW}{msg}{color.Style.RESET_ALL}")
        return kwargs


def error_upload_all(action, *args, host, **kwargs):
    if action != "fetch" and host is not None:
        msg = f"Your -ip, host contains {host}. It must be empty to use load_all!"
        print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
        exit()



# get_creds.py
import os, sys
from getpass import getpass as gp
import colorama as color

color.init()


class Creds:
    def __init__(self, *args, **kwargs):
        self.rules = None  # implement key rules here

    def set(self, msg="key", *args, force=True, confirmed=True, key=None, **kwargs):
        key = self.get(key, *args, **kwargs)
        if not key:
            key = None
            while not key:
                key = gp(prompt=f"{msg.strip(': ')}: ", stream=None)
                if force == False:
                    break
            while not confirmed:
                confirmed = self._confirm_equals(key, *args, **kwargs)
        key = self.get(key, *args, **kwargs)
        return key

    def get(self, key, *args, safeName=None, **kwargs):
        if key == "os":
            msg = f"\tUsing $env:key {safeName}"
            key = os.environ[safeName]
            print(f"{color.Fore.YELLOW}{msg}{color.Style.RESET_ALL}")
        return key

    def _confirm_equals(self, key, *args, **kwargs):
        # getting new key
        confirmKey = None
        while confirmKey != key:
            confirmKey = gp(prompt=f"re-type key to continue: ", stream=None)
        return True



# test_unittest.py

from datetime import datetime as dt
import os, re, sys
import subprocess

import joringels.src.settings as sts
import joringels.src.logger as logger


class UnitTestWithLogging:
    def __init__(self, *args, **kwargs):
        self.timeStamp = re.sub(r"([:. ])", r"-", str(dt.now()))
        self.logDir = os.path.join(sts.testPath, "logs")
        assert os.path.isdir(self.logDir), f"logDir: {self.logDir} does not exist !"
        self.logDefaultName = f"{os.path.basename(__file__)[:-3]}_{self.timeStamp}.log"
        self.log = logger.mk_logger(self.logDir, self.logDefaultName, __name__)

    def main(self, *args, **kwargs):
        with sts.temp_chdir(sts.appBasePath):
            cmds = ["python", "-m", "unittest"]
            results = (
                subprocess.Popen(cmds, stderr=subprocess.PIPE, executable=sys.executable)
                .stderr.read()
                .decode("utf-8")
            )
            results = "\n".join(
                [l for l in results.replace("\r", "").replace("\n\n", "\n").split("\n")]
            )
            self.log.info(f"\n{results}")
            for l in results.split("\n"):
                print(l)


if __name__ == "__main__":
    UnitTestWithLogging().main()



# toserver.py
import os
from joringels.src.kdbx import KeePassSecrets
from joringels.src.scp import SCPPS
from joringels.src.joringels import Joringel
from joringels.src.jorinde import Jorinde
import joringels.src.settings as sts


class Processes:
    def __init__(self, action, *args, source, method, **kwargs):
        self.action = action
        self.sources = {
            "kdbx": KeePassSecrets,
        }
        self.source = self.sources.get(source)
        self.methods = {
            "ps": SCPPS,
        }
        self.method = self.methods.get(method)

    def upload(self, *args, key, **kwargs) -> None:
        filePath = self._mk_paths(*args, **kwargs)
        s = self.source(*args, **kwargs)
        filePath, serverCreds = s.load(filePath, *args, **kwargs)
        # kdbx key is replaced by secreatskey
        kwargs.update({"key": s.secrets["key"]})
        filePath, _ = self.digest(*args, **kwargs)
        self.method(*args, **kwargs).upload(filePath, serverCreds, *args, **kwargs)

    def _mk_paths(self, *args, safeName, **kwargs) -> str:
        fileName = f"{sts.appParams.get('decPrefix')}{safeName}.yml"
        filePath = sts.prep_path(os.path.join(sts.encryptDir, fileName))
        return filePath

    def fetch(self, *args, **kwargs):
        Jorinde(*args, verbose=verbose, **kwargs).fetch(*args, **kwargs)

    def load(self, *args, **kwargs):
        KeePassSecrets(*args, **kwargs).load(*args, **kwargs)

    def serve(self, action, *args, **kwargs):
        Joringel(action, *args, **kwargs).serve(*args, **kwargs)

    def digest(self, action, *args, **kwargs):
        j = Joringel(action, *args, **kwargs)
        return j._digest(*args, **kwargs)

    def chkey(self, action, *args, **kwargs):
        Joringel(action, *args, **kwargs).chkey(*args, **kwargs)



# encryption_handler.py

import os, time, yaml
import colorama as color

color.init()
from Crypto.Cipher import AES
from Crypto.Hash import SHA256
from Crypto import Random

import joringels.src.settings as sts

# touch "decrypted_test_hint.yml" "TestJoringels: pyCallTestString"

tokenizers = {"&": "<and>", "@": "<at>"}
chunksize = 64 * 1024


class Handler:
    """handels encryption and decryption of param files
    call like:

    from joringels.src.encryption_handler import Handler as decryptor

    # h will provide:
    #                'h.decrypted':bool   - flag which confirms decryption
    #                'h.decryptPath':str  - path to decrypted file

    with decryptor(encryptPath, key) as h:
        if not h.decrypted: return False
        with open(h.decryptPath, 'r') as f:
            secrets = yaml.safe_load(f.read())
    """

    def __init__(self, encryptPath, *args, safeName="", key, retain=False, verbose=0, **kwargs):
        self.verbose = verbose
        self.decrypted = None
        self.encryptPath, self.decryptPath = self.mk_paths(encryptPath, *args, **kwargs)
        self.key = key
        self.retain = retain

    def __enter__(self, *args, **kwargs):
        """
        decrypts file and saves it to self.decryptPath
        """
        self.file_decrypt(*args, **kwargs)
        self.data_cleanup(self.decryptPath, "dec")
        return self

    def __exit__(self, *args, **kwargs):
        """
        removes decrypted file on exit
        """
        self.data_cleanup(self.decryptPath, "enc")
        encryptSuccess = None
        if self.decrypted:
            encryptSuccess = self.file_encrypt(*args, **kwargs)
        self.exit_cleanup(encryptSuccess, *args, **kwargs)

    def exit_cleanup(self, encryptSuccess, *args, **kwargs):
        if encryptSuccess == False or not self.retain:
            if os.path.isfile(self.encryptPath):
                os.remove(self.encryptPath)
        if os.path.isfile(self.decryptPath):
            os.remove(self.decryptPath)

    def mk_paths(self, secretPath: str, *args, **kwargs) -> tuple[str]:
        """
        takes a valid secretPath and creates the missing counter secretPath
        if encryptPath is given it creates the decryptPath
        if decryptPath is given it creates the encryptPath

        """
        secretsDir, secretsFileName = os.path.split(secretPath)
        if secretsFileName.startswith(sts.appParams.get("decPrefix")):
            decFileName = secretsFileName
            secretsFileName = secretsFileName.replace(sts.appParams.get("decPrefix"), "")
        decFileName = f"{sts.appParams.get('decPrefix')}{secretsFileName}"
        decryptPath = os.path.join(secretsDir, decFileName)
        encryptPath = os.path.join(secretsDir, secretsFileName)
        if (not os.path.isfile(decryptPath)) and (not os.path.isfile(encryptPath)):
            msg = f"\nsecretPath not found: {secretPath}\n"
            if self.verbose >= 2:
                print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
            raise FileNotFoundError
        elif os.path.isfile(decryptPath):
            self.decrypted = True
        else:
            self.decrypted = False
        return encryptPath, decryptPath

    def file_encrypt(self, *args, **kwargs):
        """
        ###Takes a file and encrypts its content.
        saves the encrpyted text to self.encryptPath
        """
        try:
            IV = Random.new().read(16)
            sha = SHA256.new(self.key.encode("utf-8")).digest()
            enc = AES.new(sha, AES.MODE_CBC, IV)
            with open(self.decryptPath, "rb") as inFile:
                with open(self.encryptPath, "wb") as outFile:
                    outFile.write(IV)
                    while chunk := inFile.read(chunksize):
                        if len(chunk) % 16:
                            chunk += b" " * (16 - (len(chunk) % 16))
                        outFile.write(enc.encrypt(chunk))
            return True
        except Exception as e:
            msg = f"encryption_handler.file_encrypt, Encryption failed ! : {e}"
            print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
            print(f"filePath: {self.encryptPath}")
            return False

    def file_decrypt(self, *args, **kwargs) -> bool:
        if self.decrypted:
            return None
        """
            reads a encrypted file from self.encryptPath and decrypts its content,
            saves extracted readable text to self.decryptPath file
        """
        try:
            with open(self.encryptPath, "rb") as inFile:
                IV = inFile.read(16)
                sha = SHA256.new(self.key.encode("utf-8")).digest()
                dec = AES.new(sha, AES.MODE_CBC, IV)
                with open(self.decryptPath, "wb") as outFile:
                    while chunk := inFile.read(chunksize):
                        outFile.write(dec.decrypt(chunk))
                with open(self.decryptPath, "rb") as f:
                    if fileSize := len(f.read()) == 0:
                        raise Exception(f"decrypted fileSize: {fileSize}")
            return True
        except Exception as e:
            print(f"file_decrypt, Exception: {e}")
            return False

    def data_cleanup(self, filePath: str, data: str, *args, **kwargs) -> bool:
        """
        cleans text and tokenizes decrypted file text
        notAllowed values such as & are replaced
        isValid allowes to confirm, that decryption result is readable
        """
        try:
            with open(filePath, "r+") as f:
                text = f.read()
                if (not self.decrypted) or data == "enc":
                    text = (text.strip() + sts.appParams.get("validator")).replace(
                        2 * sts.appParams.get("validator"), ""
                    )
                isValid = text.endswith(sts.appParams.get("validator"))
                for k, v in tokenizers.items():
                    text = text.replace(k, v) if data == "enc" else text.replace(v, k)
                f.seek(0)
                f.truncate()
                f.write(text.strip())
                time.sleep(0.1)
            # validate cleanup results
            if fileSize := str(len(text)).zfill(16) == 0:
                raise Exception(f"{data}: fileSize: {fileSize}")
            if data == "dec" and isValid:
                raise Exception(f"{data}: isValid: {isValid}")
            if data == "enc" and not isValid:
                raise Exception(f"{data}: isValid: {isValid}")
            self.decrypted = True
        except UnicodeDecodeError as e:
            if self.verbose:
                print(f"Decryption Failed: {e}")
            self.decrypted = False
        except Exception as e:
            print(f"data_cleanup Error with data {data}: {e}")
            self.decrypted = False



# upload.py
import os
from joringels.src.joringels import Joringel
import joringels.src.settings as sts
import importlib


def run(srcAdapt, conAdapt, action: str, *args, **kwargs) -> None:
    """
    NOTE: NON-DIGESTIVE, encrypted secretsFile remains in .ssp
    imports secrets from source, stores it in .ssp and then uploads it to remote host
    NOTE: this is only allowed on a local host computer

    run like: joringels upload -n digiserver -src kdbx -con scp
    """
    # get secret
    sec = srcAdapt.main(*args, **kwargs)

    serverCreds = sec.load(*args, **kwargs)
    # encrypt secret
    kwargs.update({"key": sec.encrpytKey})
    encryptPath, _ = Joringel(*args, **kwargs)._digest(*args, **kwargs)
    # upload to server
    conAdapt.main(*args, **kwargs).upload(serverCreds, *args, **kwargs)
    return encryptPath


def main(*args, source: str, connector: str, safeName=None, **kwargs) -> None:
    """
    imports source and connector from src and con argument
    then runs upload process using imported source an connector
    """
    assert safeName is not None, f"missing value for '-n safeName'"
    isPath = os.path.isfile(source)
    srcAdapt = importlib.import_module(
        f"{sts.impStr}.sources.{source.split('.')[-1] if isPath else source}"
    )
    conAdapt = importlib.import_module(f"{sts.impStr}.connectors.{connector}")
    # upload will temporaryly rename existing dataSafe with name identical to uploaded safe
    with sts.temp_safe_rename(*args, prefix="#upload_", safeName=safeName, **kwargs) as t:
        encryptPath = run(srcAdapt, conAdapt, *args, source=source, safeName=safeName, **kwargs)
        if os.path.exists(encryptPath):
            os.remove(encryptPath)
    return True



# jorinde.py
import colorama as color

color.init()
import os, requests, yaml
from copy import deepcopy
import joringels.src.get_soc as soc
import joringels.src.settings as sts
from joringels.src.encryption_dict_handler import text_decrypt


class Jorinde:
    def __init__(self, *args, **kwargs):
        pass

    def _fetch(self, *args, key=False, entryName=False, host=None, port=None, **kwargs):
        """<br><br>

        *Last update: 2020-11-09*
        ###Hint Secrets
        ___
        ###Asks Joringle Flower for a secret

        ########################### START TEST ###########################
        # INPUTS
        key: testkey
        entryName: TestJoringels
        encryptPath: ~/python_venvs/packages/joringels/joringels/src/test/test_get.yml

        # FUNCTION
        pyCall: instance.get(**kwargs)
        shellCall: None

        # RETURN
        returns: {'TestJoringels': 'pyCallTestString'}

        ########################### END TEST ###########################

        """
        port = sts.appParams.get("secretsPort") if port is None else port
        host = sts.dataSafeIp if host is None else host
        resp = requests.get(f"http://{host}:{port}/{entryName}")
        try:
            if resp.status_code == 200:
                secret = yaml.safe_load(resp.text)
                secret = self.clean(secret)
            else:
                secret = {"ERROR": resp.text}
        except Exception as e:
            secret = {"ERROR": e}
        return secret

    def clean(self, encrypted, *args, **kwargs):
        decrypted = {}
        for k, ciphertextBase64 in encrypted.items():
            decryptedtext = text_decrypt(ciphertextBase64)
            decrypted[k] = yaml.safe_load(decryptedtext)
        return decrypted

    def _unpack_decrypted(self, *args, safeName=None, **kwargs):
        safeName = safeName if safeName is not None else os.environ.get("DATASAFENAME")
        decPath = sts.prep_path(safeName, "unprotectedload")
        with open(decPath, "r") as f:
            entries = yaml.safe_load(f)
        # save every parameter to a seperate file
        decDir, decFileName = os.path.split(decPath)
        for entryName, prs in entries.items():
            if entryName == "key":
                continue
            else:
                if not entryName.endswith(sts.fext):
                    entryName = f"{entryName}{sts.fext}"
            with open(os.path.join(decDir, entryName), "w") as f:
                f.write(yaml.dump(prs))
        os.remove(decPath)
        msg = f"Saved entries to .ssp, NOTE: entries are unprotected !"
        print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
        return True



# upload.py
import os
from joringels.src.jorinde import Jorinde
from joringels.src.joringels import Joringel
import joringels.src.settings as sts
import importlib


def run(srcAdapt: object, action: str, *args, host, **kwargs) -> None:
    """
    imports secrets from source and unpacks it into .ssp folder
    NOTE: NON-DIGESTIVE, encrypted secretsFile remains in .ssp
    NOTE: this leaves unprotected secrets in .ssp
    NOTE: this is only allowed on a local host computer

    run like: joringels unprotectedload -n digiserver -src kdbx
    """
    sec = srcAdapt.main(*args, **kwargs)
    sec.load(*args, filePrefix=f"{action}_", **kwargs)
    Jorinde(*args, **kwargs)._unpack_decrypted(*args, **kwargs)


def main(*args, source: str, connector: str, **kwargs) -> None:
    """
    imports source
    then runs unprotected load process using imported source an connector
    """
    if os.path.isfile(source):
        moduleName = os.path.splitext(source)[-1][1:]
    else:
        moduleName = source
    srcAdapt = importlib.import_module(f"{sts.impStr}.sources.{moduleName}")
    return run(srcAdapt, *args, source=source, **kwargs)



from Crypto.Protocol.KDF import PBKDF2
from Crypto.Cipher import AES
from Crypto.Random import get_random_bytes
from Crypto.Hash import SHA256
from Crypto.Util.Padding import pad
from Crypto.Util.Padding import unpad
import base64
import os, yaml

# uses https://www.pycryptodome.org version 3.9.9


def base64Encoding(input):
    dataBase64 = base64.b64encode(input)
    dataBase64P = dataBase64.decode("UTF-8")
    return dataBase64P


def base64Decoding(input):
    return base64.decodebytes(input.encode("ascii"))


def generateSalt32Byte():
    return get_random_bytes(32)


def text_encrypt(plaintext, password=None):
    if password is None:
        password = os.environ.get("JORINGELS")
    passwordBytes = password.encode("ascii")
    salt = generateSalt32Byte()
    PBKDF2_ITERATIONS = 15000
    encryptionKey = PBKDF2(
        passwordBytes, salt, 32, count=PBKDF2_ITERATIONS, hmac_hash_module=SHA256
    )
    cipher = AES.new(encryptionKey, AES.MODE_CBC)
    ciphertext = cipher.encrypt(pad(plaintext.encode("ascii"), AES.block_size))
    ivBase64 = base64Encoding(cipher.iv)
    saltBase64 = base64Encoding(salt)
    ciphertextBase64 = base64Encoding(ciphertext)
    return saltBase64 + ":" + ivBase64 + ":" + ciphertextBase64


def text_decrypt(ciphertextBase64, password=None):
    if password is None:
        password = os.environ.get("JORINGELS")
    passwordBytes = password.encode("ascii")
    data = ciphertextBase64.split(":")
    salt = base64Decoding(data[0])
    iv = base64Decoding(data[1])
    ciphertext = base64Decoding(data[2])
    PBKDF2_ITERATIONS = 15000
    decryptionKey = PBKDF2(
        passwordBytes, salt, 32, count=PBKDF2_ITERATIONS, hmac_hash_module=SHA256
    )
    cipher = AES.new(decryptionKey, AES.MODE_CBC, iv)
    decryptedtext = unpad(cipher.decrypt(ciphertext), AES.block_size)
    decryptedtextP = decryptedtext.decode("UTF-8")
    return decryptedtextP


def main(params, password):
    # entcryption starts here
    print("SO AES CBC 256 encryption with PBKDF2 key derivation")
    encrypted = {k: text_encrypt(yaml.dump(vs), password) for k, vs in params.items()}
    print(f"{encrypted = }")
    # decryption starts here
    decrypted = {}
    for k, ciphertextBase64 in encrypted.items():
        decryptedtext = text_decrypt(ciphertextBase64, password)
        decrypted[k] = yaml.safe_load(decryptedtext)
    print(f"{decrypted = }")


if __name__ == "__main__":
    with open(os.path.join(r"C:\Users\lars\.ssp", "_joringels.yml"), "r") as f:
        params = yaml.safe_load(f)
    main(params, "6789045129812345")



"""
parses haimdall arguments
"""
import argparse


def mk_args():
    parser = argparse.ArgumentParser(description="run: python -m joringels.src.joringels info")
    parser.add_argument("action", metavar="action", nargs=None, help="see joringels_code actions")

    parser.add_argument(
        "-k",
        "--key",
        required=False,
        nargs=None,
        const=None,
        type=str,
        default=None,
        help="key to encrypt/decrypt",
    )

    parser.add_argument(
        "-nk",
        "--newKey",
        required=False,
        nargs=None,
        const=None,
        type=str,
        default=None,
        help="provide a source for new key, i.e. os",
    )

    parser.add_argument(
        "-ip",
        "--host",
        required=False,
        nargs=None,
        const=None,
        type=str,
        default=None,
        help="ip address to get secrets from",
    )

    parser.add_argument(
        "-p",
        "--port",
        required=False,
        nargs="?",
        const=7000,
        type=int,
        default=7000,
        help="port to get secrets from",
    )

    parser.add_argument(
        "-n",
        "--safeName",
        required=False,
        nargs="?",
        const=None,
        type=str,
        default=None,
        help="dataSafe title in kdbx",
    )

    parser.add_argument(
        "-e",
        "--entryName",
        required=False,
        nargs="?",
        const=None,
        type=str,
        default=None,
        help="entry title in kdbx",
    )

    parser.add_argument(
        "-src",
        "--source",
        required=False,
        nargs="?",
        const=None,
        type=str,
        default="kdbx",
        help="source of secrets, i.e. kdbx NOTE: if needed provide full path",
    )

    parser.add_argument(
        "-con",
        "--connector",
        required=False,
        nargs="?",
        const=None,
        type=str,
        default="scp",
        help="upload connector i.e. powershell scp",
    )

    parser.add_argument(
        "-v",
        "--verbose",
        required=False,
        nargs="?",
        const=1,
        type=int,
        default=0,
        help="0:silent, 1:user, 2:debug",
    )

    parser.add_argument(
        "-y",
        "--allYes",
        required=False,
        nargs="?",
        const=1,
        type=bool,
        default=None,
        help="run without confirm, not used",
    )

    parser.add_argument(
        "-rt",
        "--retain",
        required=False,
        nargs="?",
        const=True,
        type=bool,
        default=False,
        help="retain (keep) secrets after reading",
    )

    parser.add_argument(
        "-hard",
        "--hard",
        required=False,
        nargs="?",
        const=True,
        type=bool,
        default=False,
        help="PERMANENT CHANGE",
    )

    return parser.parse_args()



# standard lib imports
import colorama as color

color.init()

import os, re, shutil, sys, time
import yaml
import unittest

# test package imports
import joringels.src.settings as sts
from joringels.src.encryption_handler import Handler as Handler


# print(f"\n__file__: {__file__}")


class UnitTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls, *args, **kwargs):
        cls.verbose = 0
        cls.tempDataPath = os.path.join(sts.testDataPath, "temp")
        cls.testKey = "testKey"
        time.sleep(1)

    @classmethod
    def tearDownClass(cls, *args, **kwargs):
        try:
            shutil.rmtree(cls.tempDataPath, ignore_errors=False, onerror=None)
            # pass
        except Exception as e:
            print(f"UnitTest, tearDownClass, e: {e}")

    def mk_test_data(cls, fileDir, fileName="test_secrets", *args, **kwargs):
        if not os.path.isdir(fileDir):
            os.makedirs(fileDir)
        testFileName = f"{sts.appParams.get('decPrefix')}{fileName}.yml"
        testData = {"Joringel": "Jorinde"}
        with open(os.path.join(fileDir, testFileName), "w") as f:
            f.write(yaml.dump(testData))
        return os.path.join(fileDir, testFileName)

    def test_mk_paths(self, *args, **kwargs):
        ### finds parameter fieles by serching throu relevant joringels/app folders and packages
        testPath = self.mk_test_data(self.tempDataPath, "mk_paths", *args, **kwargs)
        inst = Handler(testPath, *args, key=self.testKey)
        self.assertEqual(
            (
                os.path.join(self.tempDataPath, "mk_paths.yml"),
                os.path.join(self.tempDataPath, "decrypted_mk_paths.yml"),
            ),
            inst.mk_paths(testPath),
        )

    def test_file_encrypt(self, *args, **kwargs):
        testPath = self.mk_test_data(self.tempDataPath, "file_encrypt", *args, **kwargs)
        inst = Handler(testPath, *args, key=self.testKey)
        inst.file_encrypt(*args, **kwargs)
        assert os.path.isfile(inst.encryptPath)
        # not working
        # with open(inst.encryptPath, 'r') as enc:
        #     self.assertRaises(UnicodeDecodeError, enc.read())

    def test_file_decrypt(self, *args, **kwargs):
        testPath = self.mk_test_data(self.tempDataPath, "file_decrypt", *args, **kwargs)
        inst = Handler(testPath, self.testKey, *args, key=self.testKey)
        inst.file_encrypt(*args, **kwargs)
        inst.file_decrypt(*args, **kwargs)
        with open(inst.decryptPath, "r") as dec:
            self.assertEqual(dec.read().strip(), "Joringel: Jorinde")


if __name__ == "__main__":
    unittest.main()
    print("done")
    exit()



# ps_upload.py

import os, sys, getpass
import subprocess
import joringels.src.settings as sts
import joringels.src.get_soc as soc


class SCPPS:
    def __init__(self, *args, **kwargs):
        pass

    def upload(self, serverCreds, localPath=None, *args, safeName, **kwargs):
        localPath = localPath if localPath is not None else sts.prep_path(safeName)
        if os.name == "nt":
            bash = "powershell.exe"
            script = os.path.join(sts.settingsPath, "connectors", "scp.ps1")

        cmds = [
            bash,
            script,
            serverCreds["rmUserName"],
            serverCreds["rmHost"],
            localPath,
            serverCreds["rmPath"].replace(f"C:\\Users\\{getpass.getuser()}", "~"),
            serverCreds["rmKey"],
        ]
        p = subprocess.Popen(cmds, stdout=sys.stdout)
        p.communicate()


def main(*args, **kwargs):
    scp = SCPPS(*args, **kwargs)
    return scp



# get_secrets.py

from datetime import datetime as dt
from pykeepass import PyKeePass as keePass
import os, re, yaml
import colorama as color
from joringels.src.get_creds import Creds

color.init()

import joringels.src.settings as sts

# :)L0veMi11i0n$
class KeePassSecrets:
    def __init__(self, action, *args, safeName, key=None, **kwargs):
        self.groups, self.safeName = {}, safeName
        self.secrets, self.secretsKey, self.serverCreds = {}, "", {}
        self.kPath = self._check_kPath(*args, **kwargs)
        self.creds = Creds(*args, **kwargs).set("KeePass login", *args, **kwargs)
        self.session = keePass(self.kPath, self.creds)
        self.dataSafes = self.session.find_groups(name=sts.groupName, first=True)
        self.dataSafe = self.session.find_entries(title=safeName, group=self.dataSafes, first=True)
        if action != "show":
            self.targets, self.entries = self._get_safe_params(*args, **kwargs)

    def _check_kPath(self, *args, source, **kwargs):
        kPath = sts.appParams.get("kPath", source)
        if not os.path.isfile(kPath):
            kPath = os.path.expanduser(os.environ.get("secrets", kPath))
        if not os.path.isfile(kPath):
            msg = (
                f"kPath is not a file: {kPath}! "
                f"If sts.appParams['kPath'] is not existing, provide a full "
                f"path/to/file.kdbx !"
            )
            print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
            exit()
        return kPath

    def _get_safe_params(self, *args, **kwargs) -> list:
        if self.dataSafe is None:
            msg = f"keepass._get_safe_params with data_safe not found: {self.safeName}"
            print(f"{color.Fore.RED}{msg}{color.Style.RESET_ALL}")
            return None, None
        self.encrpytKey = self.dataSafe.password
        attachments = self._get_attachments(self.dataSafe)
        safe_params = attachments.get(sts.safeParamsFileName)
        self.joringelsParams = attachments.get(sts.appParamsFileName)
        targets = dict([reversed(os.path.split(p)) for p in safe_params["targets"]])
        entries = safe_params["entries"]
        return targets, entries

    def _mk_secrets(self, *args, **kwargs):
        for entryPath in self.entries:
            groupPath, entryName = os.path.split(entryPath)
            group = self.session.find_groups(path=groupPath)
            entry = self.session.find_entries(title=entryName, group=group, first=True)
            if entry is None:
                print(f"keepass._extract_entries, entry not found: {entry}")
            self.secrets[entry.title] = {
                "key": self.encrpytKey,
                "title": entry.title,
                "username": entry.username,
                "password": entry.password,
                "url": entry.url,
            }
            if entry.attachments:
                self.secrets[entry.title].update(self._get_attachments(entry, *args, **kwargs))

    def _mk_server_params(self, target, host, *args, **kwargs):
        group = self.session.find_groups(path=target)
        entry = self.session.find_entries(title=host, group=group, first=True)
        self.serverCreds["rmUserName"] = entry.username
        self.serverCreds["rmKey"] = entry.password
        self.serverCreds["rmHost"] = entry.url
        self.serverCreds["rmPath"] = sts.encryptDir

    def _get_attachments(self, entry, *args, **kwargs):
        attachs = {}
        for a in entry.attachments:
            try:
                attachs[a.filename] = yaml.safe_load(a.data)
            except Exception as e:
                print(f"keepass._get_attachments: {e}")
        return attachs

    def _write_secs(self, *args, safeName, filePrefix=None, **kwargs):
        filePrefix = filePrefix if filePrefix else sts.appParams.get("decPrefix")
        fileName = f"{filePrefix}{safeName}.yml"
        filePath = sts.prep_path(os.path.join(sts.encryptDir, fileName))

        # file extension is .yml
        with open(filePath, "w") as f:
            f.write(yaml.dump(self.secrets))

    def load(self, *args, host=None, **kwargs) -> None:
        host = host if host is not None else list(self.targets)[0]
        target = self.targets.get(host, None)
        self._mk_server_params(target, host, *args, **kwargs)
        self._mk_secrets(*args, **kwargs)
        self.secrets[sts.appParamsFileName] = self.joringelsParams
        self._write_secs(*args, **kwargs)
        return self.serverCreds

    def show(self, *args, safeName: str, **kwargs) -> None:
        """
        gets all relevant entry paths from keepass and prints them in a copy/paste
        optimized way

        run like:   python -m joringels.src.sources.kdbx show -n python_venvs
                    enter keepass key when prompted
        copy the entries into the NOTES of you keepass joringels_data_save entry

        NOTE: Each safe needs one server login credential entry for upload
            server login credential start like: !~/python_venvs/.../...
            normal entries look like:             python_venvs/.../...
        """
        for i, element in enumerate(self.session.find_entries(title=".*", regex=True)):
            if element.path[0] == sts.entriesRoot:
                entryPath = sts.kps_sep.join(element.path)
                print(f"{i} copy to Notes:\t{entryPath}")


def main(action=None, *args, **kwargs):
    inst = KeePassSecrets(action, *args, **kwargs)
    if action is None:
        return inst
    else:
        return getattr(inst, action)(*args, **kwargs)


if __name__ == "__main__":
    import joringels.src.arguments as arguments

    kwargs = arguments.mk_args().__dict__
    keepass = main(**kwargs)



# get_soc.py -> import joringels.src.get_soc as soc

import os, socket
import joringels.src.settings as sts


def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    socName = s.getsockname()[0]
    return socName


def get_hostname():
    return socket.gethostname().upper()


def get_allowed_hosts(*args, **kwargs):
    allowedHosts = sts.appParams.get("allowedHosts")
    if get_hostname() in sts.appParams.get("secureHosts"):
        allowedHosts.append(get_ip())
    return allowedHosts


def host_info(*args, host=False, port=False, **kwargs):
    host = host if host else get_ip()
    port = port if port else sts.appParams.get("secretsPort")
    return host, port



# logger.py
import os, re
import logging
from datetime import datetime as dt

import joringels.src.settings as sts


def mk_logger(logDir, fileName, loggerName, *args, **kwargs):
    # logging config to put somewhere before calling functions
    # call like: logger.debug(f"logtext: {anyvar}")
    logger = logging.getLogger(loggerName)
    logger.setLevel(logging.INFO)
    logformat = "%(asctime)s %(levelname)s %(module)s - %(funcName)s: %(message)s"
    datefmt = "%m-%d %H:%M"
    logForm = logging.Formatter(fmt=logformat, datefmt=datefmt)
    logPath = os.path.join(logDir, fileName)
    logHandler = logging.FileHandler(logPath, mode="a")
    logHandler.setFormatter(logForm)
    logger.addHandler(logHandler)
    return logger


def log(name, msg, *args, verbose=0, **kwargs):
    timeStamp = re.sub(r"([:. ])", r"-", str(dt.now()))
    if verbose >= 1:
        print(msg)
    log = mk_logger(sts.logDir, f"{timeStamp}_{name}.log", name)
    log.info(msg)
